<?php
/*
 * HostDEV Module name
 *
 * @Version 1.0
 * @author Eugeny Panikarowsky - evgenii_panikaro@mail.ru
 * @copyright © year Eugeny Panikarowsky
 * @url https://hostdev.pw/
 *
 */

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * HostDEV Field Helper
 *
 * @Version 1.0
 * @author Eugeny Panikarowsky - evgenii_panikaro@mail.ru
 * @copyright © 2022 Eugeny Panikarowsky
 * @url https://hostdev.pw/
 *
 */

class HostDev_FieldHelper_Controller
{

	/**
	 * Пользовательские поля
	 * @var array
	 */
	static protected $_fields = [];

	public static function _setEvents()
	{

		if (Core::moduleIsActive('hostdev_fieldhelper')) {
			$aAllFields = self::_getFields();
			foreach ($aAllFields as $sModelName => $aFields) {
				if (count($aFields)) {
					foreach ($aFields as $oField) {
						Core_Event::attach($sModelName . '.onCalluf_' . $oField->tag_name, array('HostDev_FieldHelper_Controller', 'onCall'), $oField);
					}
					Core_Event::attach($sModelName . '.onAfterSave', array('HostDev_FieldHelper_Controller', 'onSave'));
					Core_Event::attach($sModelName . '.onBeforeGetXml', array('HostDev_FieldHelper_Controller', 'onBeforeGetXml'), $aFields);
				}
			}
		}
	}

	/**
	 * @return array
	 */
	protected static function _getFields()
	{
		if (!empty(self::$_fields)) {
			return self::$_fields;
		}
		$oFields = Core_Entity::factory('Field');
		$oFields->getTableColumns();
		$oFields->queryBuilder()
				->open()
				->where('fields.site_id', '=', CURRENT_SITE)
				->setOr()
				->where('fields.site_id', '=', 0)
				->close()
				->clearOrderBy()
				->orderBy('sorting', 'ASC');

		/* @var $oField Field_Model */
		$aTmpFields = $oFields->findAll();

		$aFields = array();
		foreach ($aTmpFields as $oField) {
			$aFields[$oField->model][$oField->tag_name] = $oField;
		}

		return self::$_fields = $aFields;
	}

	private static function _correctValue($oField, $value)
	{
		switch ($oField->type) {
			case 0: // Int
			case 7: // Checkbox
			case 3: // List
				$value = (int)$value;
				break;
			case 1: // String
			case 4: // Textarea
			case 6: // Wysiwyg
				$value = (string)$value;
				break;
			case 11: // Float
				$value = (float)str_replace(',', '.', $value);
				break;
			case 8: // Date
				$value = $value == ''
						? '0000-00-00 00:00:00'
						: Core_Date::date2sql($value);
				break;
			case 9: // Datetime
				$value = $value == ''
						? '0000-00-00 00:00:00'
						: Core_Date::datetime2sql($value);
				break;
		}

		return $value;
	}

	public static function onBeforeGetXml(Core_Entity $oObject, $args, $aFields = array()) {
		if (is_array($aFields) && count($aFields)) {
			foreach ($aFields as $oField) {
				$oObject->addForbiddenTags(array('cuf_'.$oField->tag_name, 'uf_'.$oField->tag_name));
			}
		}
		//$oObject->addForbiddenTag();
	}

	public static function onSave(Core_Entity $oObject)
	{
		$Reflection = new ReflectionProperty('Core_Orm', '_dataValues');
		$Reflection->setAccessible(true);
		$aDataValues = $Reflection->getValue($oObject);

		$sPrimaryKey = $oObject->getPrimaryKey();
		$sModelName = $oObject->getModelName();
		if (count($aDataValues)) {
			foreach (self::$_fields[$sModelName] as $sTagName => $oField) {
				if (isset($aDataValues['cuf_'.$sTagName])) {
					$aValues = $aDataValues['cuf_'.$sTagName];
					$aFieldValues = $oField->getValues($sPrimaryKey);
					if (is_array($aValues)) {
						foreach ($aValues as $sValue) {
							if (isset($aFieldValues[0])) {
								$oValue = array_shift($aFieldValues);
							} else {
								$oValue = $oField->createNewValue($sPrimaryKey);
							}
							$oValue->value(self::_correctValue($oField, $sValue));
							$oValue->save();
						}
					} else if (!is_null($aValues)) {
						if (isset($aFieldValues[0])) {
							$oValue = array_shift($aFieldValues);
						} else {
							$oValue = $oField->createNewValue($sPrimaryKey);
						}

						$oValue->setValue(self::_correctValue($oField, $aValues));
						$oValue->save();
					}
					if (count($aFieldValues)) {
						foreach ($aFieldValues as $oValue) {
							$oValue->delete();
						}
					}
				}
			}
		}
	}

	/**
	 * @param Core_Entity $oObject
	 * @param array $args
	 * @param Field_Model $oField
	 * @return array|mixed|string
	 */
	public static function onCall(Core_Entity $oObject, array $args, Field_Model $oField)
	{

		$Reflection = new ReflectionProperty('Core_Orm', '_dataValues');
		$Reflection->setAccessible(true);
		$aDataValues = $Reflection->getValue($oObject);

		if (count($args)) {
			$aDataValues['cuf_'.$oField->tag_name] = $aDataValues['uf_'.$oField->tag_name] = $args[0];
			$Reflection->setValue($oObject, $aDataValues);
		} else if (isset($aDataValues['uf_'.$oField->tag_name])) {
			return $aDataValues['uf_'.$oField->tag_name];
		} else {
			$pkv = $oObject->getPrimaryKey();
			$aFieldValues = $oField->getValues($pkv);
			$sResult = '';
			if (count($aFieldValues)) {
				if (count($aFieldValues) > 1) {
					$sResult = array_map(static function ($oValue) {
						return $oValue->value;
					}, $aFieldValues);
				} else {
					$sResult = $aFieldValues[0]->value;
				}
				$aDataValues['uf_'.$oField->tag_name] = $sResult;
				$Reflection->setValue($oObject, $aDataValues);
				return $sResult;
			}
		}
	}

}